/*
 * ao.c - Milena Audiobook Creator
 * Copyright (C) Bohdan R. Rau 2009-2012 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */
#include "config.h"
#include "gmilena.h"
#include <ao/ao.h>
#include <string.h>
#ifdef HAVE_PULSE
#include <pulse/simple.h>
#include <pulse/error.h>
#endif

int use_pulse=0;
static int internal_use_pulse;
int pulse_margin=300;

int audio_close_device(void);

static int playing;

static ao_device *device;
static int driver=-1;
static ao_sample_format format;
static int ao_initialized;
int dont_close_audiodevice;

static int init_ao(int rate)
{
	format.bits = 16;
	format.channels = 1;
	format.rate = rate;
	format.byte_format = AO_FMT_LITTLE;

	if (!ao_initialized) ao_initialize();
	if (driver == -1) {
		driver = ao_default_driver_id();
		if (driver == -1) {
			return 0;
		}
	}
	if (!device) device = ao_open_live(driver, &format, NULL);
	if (!device) {
		return 0;
	}
	return 1;
}


#ifdef HAVE_PULSE

pa_simple *p_simple;
int pulse_samples_sent;

static int mypa_init(int rate)
{
	pa_buffer_attr buffAttr;
	pa_sample_spec ss;
	int error;

	ss.rate = rate;
	ss.channels = 1;
	ss.format = PA_SAMPLE_S16LE;

	format.rate = rate;

	buffAttr.maxlength = (uint32_t)-1;
	buffAttr.tlength = 100;
	buffAttr.prebuf = (uint32_t)-1;
	buffAttr.minreq = (uint32_t)-1;
	buffAttr.fragsize = (uint32_t)-1;
	p_simple = pa_simple_new(
				NULL,
				"Milena",
				PA_STREAM_PLAYBACK,
				     NULL, "playback", &ss, NULL, &buffAttr, &error);
	if (!p_simple) {
		ErrorF(NULL,"PulseAudio","New: %s",pa_strerror(error));
		return 0;
	}
	pulse_samples_sent=0;
	return 1;
}

static void mypa_close(void)
{
	if (!p_simple) return;
	if (internal_use_pulse) {
		short pbuffer[1024];
		int nc=pulse_margin * (format.rate/1000);
		int m;
		memset(pbuffer,0,sizeof(pbuffer));
		while (nc > 0) {
			m=1024;
			if (m > nc) m = nc;
			nc -= m;
			pa_simple_write(p_simple,(char *)pbuffer,2*m,NULL);
		}
	}
	pa_simple_free(p_simple);
	p_simple=0;
}

#endif

static int init_audio(int rate)
{
#ifdef HAVE_PULSE
		internal_use_pulse=use_pulse;
		if (use_pulse) {
			return mypa_init(rate);
		}
#endif
	return init_ao(rate);
}

int audio_play_sfr(short *samples,int nsamples,int rate)
{
	if (!samples) {
		if (playing) {
			audio_close_device();
		}
		return 1;
	}
    if (playing && rate != format.rate) {
		audio_close_device();
    }
	if (!playing) {
		if (!init_audio(rate)) return 0;
		playing=1;
	}
	int rc;
#ifdef HAVE_PULSE
	if (internal_use_pulse) {
		int error;
		rc = pa_simple_write(p_simple, (char *)samples, 2*nsamples, &error) >= 0;
		if (!rc) {
			ErrorF(NULL,"PulseAudio","Write: %s",pa_strerror(error));
		}
	}
	else
#endif
	rc=ao_play(device,(char *)samples,2*nsamples);

	if (!rc) {
		audio_close_device();
		return 0;
	}
	return 1;
}

int audio_play(short *samples,int nsamples)
{
	return audio_play_sfr(samples,nsamples,16000);
}

int audio_close_device(void)
{
	playing=0;
#ifdef HAVE_PULSE
	if (use_pulse) {
		mypa_close();
		return 1;
	}
#endif
	if (device) {
		ao_close(device);
		device=NULL;
	}
    return 1;
}
