/*
 * forvo.c - Milena Audiobook Creator
 * Copyright (C) Bohdan R. Rau 2009-2011 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */

#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "gmilena.h"
#ifdef HAVE_CURL
#include <curl/curl.h>
#endif
#include <ctype.h>

static void howjsay_speak(char *url);

static void append_lc_urlenc(GString *gs,char *str,int iplus)
{
	for (;*str;str++) {
		if (*str==' ' && iplus) {
			g_string_append_c(gs,'+');
		}
		else if (isalnum(*str) || strchr(".-_",*str)) {
			g_string_append_c(gs,*str);
		}
		else {
			g_string_append_printf(gs,"%%%02X",(*str) & 255);
		}
	}
}

void gotoforvo(void *widget, void *data)
{
	char *str=NULL,*str2,*instr;
	GString *forvo;
	GError *err=NULL;
	int ced=get_current_editor();
    if (ced == MILEDIT_MODE_DIC) {
        str=get_diction_line(0,1);
    }
    else if (ced == MILEDIT_MODE_NORMAL) {
        str=get_word_at_cursor(tresc,0);
    }
    if (!str) return;
    str2=g_utf8_strdown(str,-1);
    free(str);
    str=str2;
	if (!str) return;
	instr="http://www.forvo.com/search/";
	if (data && !strncmp((char *)data,"ho",2)) {
		instr="http://www.howjsay.com/index.php?word=";
	}
	forvo=g_string_new(instr);
	append_lc_urlenc(forvo,str,1);
	g_free(str);
	str=g_string_free(forvo,0);
#ifdef USE_WEBPRON
	if (data && !strncmp(data,"ho",2)) {
		start_busy_cursor();
		howjsay_speak(str);
		end_busy_cursor();
		g_free(str);
		return;
	}

#endif
#ifdef USE_WEBKIT
    browser(str);
#else
	gtk_show_uri(NULL,str,GDK_CURRENT_TIME,&err);
#endif
	g_free(str);
	if (err) {
		Error("Błąd wywołania URL",err->message);
		g_error_free(err);
	}
}




#ifdef USE_WEBPRON

#ifdef HAVE_CURL

static int curl_error=1;

void initialize_curl(void)
{
	curl_error=curl_global_init(0);
}

static char *curl_data;
static int curl_len;


size_t curl_write(void *ptr,size_t size, size_t nmemb, void *userdata)
{
	size_t len=size * nmemb;
	if (!len) return 0;
	if (!curl_data) {
		curl_data=malloc(len);
		curl_len=0;
	}
	else curl_data=realloc(curl_data,curl_len+len);
	memcpy(curl_data+curl_len,ptr,len);
	curl_len+=len;
	return len;
}

int curl_fetch_file(char *url,void **data,int *len)
{
	CURL *curl;
	int rc=-1;

	if (curl_error) return -1;
	curl=curl_easy_init();
	curl_data=NULL;
	curl_len=0;
	curl_easy_setopt(curl,CURLOPT_NOSIGNAL,1);
	curl_easy_setopt(curl,CURLOPT_TIMEOUT,10);
	curl_easy_setopt(curl,CURLOPT_FOLLOWLOCATION,1);
	curl_easy_setopt(curl,CURLOPT_MAXREDIRS,5);
	curl_easy_setopt(curl,CURLOPT_WRITEFUNCTION,curl_write);
	curl_easy_setopt(curl,CURLOPT_WRITEDATA,NULL);
	curl_easy_setopt(curl,CURLOPT_URL,url);
	curl_easy_setopt(curl,CURLOPT_USERAGENT,"Mozilla/5.0 (compatible; Milena AudioBook Creator "PACKAGE_VERSION")");
	rc=curl_easy_perform(curl);
	if (!rc) {
		long rcode=0;
		curl_easy_getinfo(curl,CURLINFO_RESPONSE_CODE,&rcode);
		if (rcode != 200) {
			if (!rcode) rc=-1;
			else rc=rcode;
		}
	}
	if (rc) {
		if (curl_data) free(curl_data);
		curl_data=NULL;
		curl_len=0;
	}
	curl_easy_cleanup(curl);
	*data=curl_data;
	*len=curl_len;
	curl_data=NULL;
	return rc;
}

#endif

static struct webpron_cache {
	struct webpron_cache *next;
	char *url;
	int length;
	char *body;
} * webpron;

static void howjsay_speak(char *url)
{
	char *body;int len,rc;
	char *swrd,*c;
	GString *fvs;
	struct webpron_cache *wb;
	for (wb=webpron;wb;wb=wb->next) if (!strcmp(wb->url,url)) break;
	if (wb) {
		body=wb->body;
		if (!body) {
			Error("Błąd","Brak słowa w serwisie howjsay.com");
			return;
		}
		len=wb->length;
		goto ohayo;
	}
#ifdef HAVE_CURL
	rc=curl_fetch_file(url,(void **)&body,&len);
#else
	body=wget(url,&len);
	rc=body == NULL;
#endif
	if (rc) {
		Error("Błąd","Pobranie strony się nie udało");
		return;
	}
	if (!body) {
		Error("Błąd","Strona jest pusta");
		return;
	}
	wb=malloc(sizeof(*wb));
	wb->next=webpron;
	webpron=wb;
	wb->url=strdup(url);
	wb->body=NULL;
	wb->length=0;
	body=realloc(body,len+1);
	body[len]=0;
	if (strstr(body,"nearest entry is")) swrd=NULL;
	else swrd=strstr(body," flashvars=\"word=");
	if (!swrd) {
		free(body);
		Error("Błąd","Brak słowa w serwisie howjsay.com");
		return;
	}
	swrd+=17;
	c=strchr(swrd,'&');if (c) *c=0;
	c=strchr(swrd,'=');if (c) *c=0;
	fvs=g_string_new("http://www.howjsay.com/mp3/");
	append_lc_urlenc(fvs,swrd,0);
	g_string_append(fvs,".mp3");
	free(body);
	body=NULL;
	c=g_string_free(fvs,0);
#ifdef HAVE_CURL
	rc=curl_fetch_file(c,(void **)&body,&len);
#else
	body=wget(c,&len);
	rc=body == NULL;
#endif

	g_free(c);
	if (rc) {
		Error("Błąd","Nie mogłem pobrać wymowy z howjsay.com");
		return;
	}
	wb->body=body;
	wb->length=len;
ohayo:
	mp3_play(body,len);
	//free(body);
}
#endif

